#!/bin/bash
#
# cert_manage.sh - Certificate Management script
#
# Type  : Utility
#
# Author: Rob Allen
#
# Date  : 16.10.2008
#
# Program Description
# ===================
# Script to import and manage SSL certificates
# Supported suppliers: comodo
#
#
# Amendment History
# =================
# 17.10.08 Created today (RA)
# 17.10.08 Basic script to import certificates into Appliance 2 (RA)
# 31.10.08 Major changes to all aspects of script (RA)
# 20.02.09 Major changes following review and design meeting (RA)
# 07.07.09 Added script version for smenu.sh to picker (RA)
# 22.07.09 Changed copyright date, and OU and O around (RA)
# 28.07.09 Changed certificate import routine. Added clone facility. Updated
#          view options. Changed Selfsigned certificate alias (RA)
# 29.07.09 Changed named of script, previous inaccurate name (RA)
# 30.03.10 Fixed spelling mistake (RA)
# 30.03.10 Changed 'import_cert' to remove alias check, and the forced del (RA)
# 31.03.10 Facility to select key size on creation of local cert. (RA)
# 06.12.10 Added check to ensure alias names are valid during clone process (RA)
# 06.12.10 Added option to choose both a cert and a clone destionation name (RA)
# 06.12.10 Removed certificate deletion from the clone routine (RA)
# 06.12.10 Added 'yorn' to delete cert process (RA)
# 20.01.12 Removed alias exist check - You must have the ability to import reply crt into same alias (RB)
# 25.01.12 updated the copyright year as requested by Chris. (RB)
#

KEYSTORE=/home/swivel/.keystore
DIR=/backups/upload
inv=`tput smso`
nor=`tput rmso`
PINSAFE_TYPE="$1"
KEYDIR=/usr/java/latest/bin/
PASSWORD=lockbox
version="0.10"
COPYRIGHT_YEAR="2012"

initialise() {
 if test ! -d $DIR; then
  mkdir -p $DIR
  chmod 777 $DIR; fi
}

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo 
 echo $1
 echo
}

certs() {
 heading "Certificate Management"
 echo "1. Create a local Certificate"
 echo "2. Generate CSR"
 echo "3. Import Certificate, Root CA or Intermediate"
 echo "4. View keystore"
 echo "5. Delete Certificate from Keystore"
 echo "6. Generate a Self Signed Certificate"
 echo "7. Clone certificate"
 echo "0. Exit"
 echo
 read -p "Select: " option
 case $option in
  1) create_local_cert; certs; ;;
  2) create_csr; certs; ;;
  3) import_cert; certs; ;;
  4) view_keystore; certs; ;;
  5) del_cert; certs; ;;
  6) gen_self_signed; certs; ;;
  7) clone_cert; certs; ;;
  *) exit; ;;
 esac
}

view_keystore() {
 heading "View Keystore"
 list_keystore_command | grep "Alias name"
 echo
 read -p "Enter Certificate name to view, or Return for all Certificates: " alias_name
 if test $alias_name; then
 # check this $ bug Alias name: $alias_name$
  if test "$(list_keystore_command | grep "Alias name: $alias_name$")"; then
   list_keystore_command $alias_name | grep -v "^Keystore" | grep -v "^*******************************************$"|more
  else
   echo
   echo "Please select a valid certificate name"
  fi
 else
  list_keystore_command $yn | grep -v "^Keystore" | grep -v "^*******************************************$"|more
 fi
 echo
 read -p "Press return to Continue"
}

list_keystore_command() {
 if test $1; then
  $KEYDIR/keytool -list -keystore $KEYSTORE -v -storepass $PASSWORD -alias $1
 else
  $KEYDIR/keytool -list -keystore $KEYSTORE -v -storepass $PASSWORD
 fi
}

clone_cert() {
 heading "Clone Certificate"
 list_keystore_command | grep "Alias name"
 echo
 read -p "Enter Certificate name to clone: " alias_name
 if test $alias_name; then
  if test "$(list_keystore_command | grep "Alias name: $alias_name$")"; then
   echo; read -p "Enter Certificate alias name for the clone: " clone_name
   if test $clone_name; then
    if test "$(list_keystore_command | grep "Alias name: $clone_name$")"; then
     echo; echo "Please select an alias name that is not already used"
    else
     clone_selected_cert $alias_name $clone_name
    fi
   else
    echo; echo "A certificate clone Alias name is required."
   fi
  else
   echo; echo "Please select a valid certificate name"
  fi
 else
  echo; echo "Please select a valid certificate name"
 fi

 read -p "Press return to Continue"
}

clone_selected_cert() {
 echo
 echo "Continuing will clone the \"$1\" certificate to \"$2\""
 yorn certs
 $KEYDIR/keytool -keyclone -alias $1 -dest $2 -keystore $KEYSTORE -storepass $PASSWORD -keypass $PASSWORD -new $PASSWORD
 echo; echo "Certficate cloned"
 echo
}

del_cert() {
 heading "Delete Keystore"
 list_keystore_command | grep "Alias name"
 echo
 read -p "Enter certificate Alias to delete: " alias_name
 if test $alias_name; then
  if test "$(list_keystore_command | grep "Alias name: $alias_name$")"; then
   yorn certs
   echo
   cp $KEYSTORE $KEYSTORE.`date +%d%m%y_%s`
   $KEYDIR/keytool -delete -keystore $KEYSTORE -storepass $PASSWORD -alias $alias_name
  else
   echo; echo "Please select a valid certificate name"
  fi
 else
  echo; echo "A certificate Alias name is required."
 fi
 echo; read -p "Press Return to Continue" yn
}

example_local_cert() {
 echo "${inv}Example:${nor}"
 echo "Domain Name (CN)  : pinsafe.swivelsecure.com"
 echo "Company Name (O)  : Swivel Secure Ltd"
 echo "Department (OU)   : IT Department"
 echo "City (L)          : Wetherby"
 echo "County (ST)       : West Yorkshire"
 echo "Country Code (C)  : GB"
 echo; echo
}

local_cert_details() {
 example_local_cert
 read -p "Domain Name   : " domain
 read -p "Company Name  : " company_name
 read -p "Department    : " department
 read -p "City          : " city
 read -p "County        : " county
 read -p "Country Code  : " country_code
 echo

 if test ! "$domain" || test ! "$company_name" || test ! "$department" || test ! "$city" || test ! "$county" || test ! "$country_code"; then
  echo
  echo "All information must be provided in order to build a certificate"
  read -p "Press Return to Continue"
  certs; fi
}

disp_local_cert_details(){
 echo "Domain Name   : " $domain
 echo "Company Name  : " $company_name
 echo "Department    : " $department
 echo "City          : " $city
 echo "County        : " $county
 echo "Country Code  : " $country_code
 echo
}

gen_self_signed() {
 heading "Generate Self Signed Certificate"
 if test "$(list_keystore_command | grep "Alias name: selfsigned$")"; then
  echo
  echo "Please delete the existing certificate, before generating a new one."
 else
  local_cert_details
  heading "Create Local certificate"
  disp_local_cert_details
  cp $KEYSTORE $KEYSTORE.`date +%d%m%y_%s`
  $KEYDIR/keytool -genkey -validity 3600 -alias selfsigned -keyalg RSA -keystore $KEYSTORE -storepass $PASSWORD -keypass $PASSWORD -dname "CN=$domain, O=$company_name, OU=$department, L=$city, ST=$county, C=$country_code"

  chmod 600 $KEYSTORE
  chown -R swivel:swivel $KEYSTORE
  echo; echo "Self Signed Certificate created."
 fi
 echo; read -p "Press Return to Continue" yn
}

create_local_cert() {
 heading "Create Local certificate"
 echo -n "Checking keystore..."
 if test "$(list_keystore_command | grep "Alias name: swivel$")"; then
  echo
  echo "Please delete/clone the existing certificate, before generating a new one."
 else
  get_key_size
  local_cert_details
  heading "Create Local certificate"
  disp_local_cert_details
  cp $KEYSTORE $KEYSTORE.`date +%d%m%y_%s`
  $KEYDIR/keytool -keysize $key_size -genkey -dname "CN=$domain, O=$company_name, OU=$department, L=$city, ST=$county, C=$country_code" -alias swivel -keyalg RSA -storepass $PASSWORD -keypass $PASSWORD -keystore $KEYSTORE
  chmod 600 $KEYSTORE
  chown -R swivel:swivel $KEYSTORE
  echo; echo "Local Certificate created."
 fi
  echo; read -p "Press Return to Continue"
}

get_key_size() {
 heading "Create Local certificate"
 echo "Certificate Key size"
 echo
 echo "1. 1024"
 echo "2. 2048"
 echo "3. 4096"
 echo "0. Exit"
 echo
 read -p "Select: " option
 case $option in
  1) key_size=1024; ;;
  2) key_size=2048; ;;
  3) key_size=4096; ;;
  0) certs; ;;
  *) get_key_size; ;;
 esac
}

create_csr() {
 heading "Generate CSR"
 list_keystore_command | grep "Alias name"
 echo
 read -p "Enter Certificate name to create CSR for: " alias_name
 if test $alias_name; then
  if test "$(list_keystore_command | grep "Alias name: $alias_name$")"; then
   $KEYDIR/keytool -certreq -keyalg RSA -alias $alias_name -file $DIR/$alias_name.csr -storepass $PASSWORD -keystore $KEYSTORE
 echo "CSR created in $DIR/$alias_name.csr"
  else
   echo; echo "Please select a valid certificate name"
  fi
 else
  echo; echo "Please select a valid certificate name"
 fi
 echo; read -p "Press Return to Continue"
}

import_cert() {
# install the SSL Certificate
 heading "Import Certificate"
 echo "   ####################################################################"
 echo "   ## Upload your Certificate response, Root CA or Intermediate file ##"
 echo "   ## to /backups/upload                                             ##"
 echo "   ####################################################################"
 echo; read -p "Enter the filename of the file you uploaded: " filename
 if test $filename; then
  if test -f $DIR/$filename; then
   read -p "Enter a unique alias name: " alias_name
   if test $alias_name; then
# You must have the ability to import reply crt into same alias.
#    if test ! "$(list_keystore_command | grep "Alias name: $alias_name$")"; then
     yorn certs
     cp $KEYSTORE $KEYSTORE.`date +%d%m%y_%s`
     $KEYDIR/keytool -import -alias $alias_name -keystore $KEYSTORE -trustcacerts -file $DIR/$filename -storepass $PASSWORD
#    else
#     echo; echo "Use a unique alias name, \"$alias_name\" already used."
#    fi
   else
    echo; echo "Alias name, cannot be left empty."
   fi
  else
   echo; echo "The certificate file \"$DIR/$filename\" could not be found."
  fi
 else
  echo; echo "Filename, cannot be left empty."
 fi
 echo; read -p "Press Return to Continue." yn
}

yorn() {
 echo
 read -p "Are you sure? (Yes/No): " yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y"; then
  $1; fi
}

initialise
certs
