#!/bin/bash
#
# update.sh - Appliance Update script
#
# Type  : Utility
#
# Author: Rob Allen
#
# Date  : 12.01.2009
#
# Program Description
# ===================
# Script to manage and control update process
#
#
# Amendment History
# =================
# 12.01.09 Created today (RA)
# 09.07.09 Changed patch_count function, and invoke it directing output to null
# 05.02.10 Changed del patch command to remove directories.
# 25.01.12 Updated the copyright year as requested by Chris. (RB)
# 24.08.12 Changed the calling routine for install.sh to pass screen display
#	   over to the patch, removes wrapping issues based on version name (RA)
#          

inv=`tput smso`
nor=`tput rmso`
UPLOAD=/backups/upload
PATCHBASE=/backups/patch
LOCK=/var/lock/subsys
TOMCAT=/usr/local/tomcat/webapps
TOMCAT_CONF=$TOMCAT/pinsafe/WEB-INF/conf
PINSAFE_TYPE="$1"
LocalhostIP="`grep Primary_IP /etc/mon/mon.cf|awk '{ print $3}'`"
COPYRIGHT_YEAR="2012"
version="0.10"

initialise() {
 if test ! -d $UPLOAD; then
  mkdir $DIR; fi
 if test ! -d $PATCHBASE; then
  mkdir $PATCHBASE; fi
 if test ! -d $backups/config; then
  mkdir -p /backups/config; fi
}

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo 
 echo $1
 echo
}

s_heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo
}

menu() {
 heading "Patch/Update management"

 if test -f $PATCHBASE/README; then
  sub_menu patch; fi  

 if `ls $UPLOAD/[pP]atch.* > /dev/null 2>&1`; then
  echo "1. Install new patch file"
  echo "0. Exit"
  echo
  read -p "Select: " option
  case $option in
   1) sub_menu patch; menu; ;;
   *) exit; ;;
  esac
 fi
}

sub_menu() {
 heading "Patch/Update management"

 if test $1 = "patch"; then
  if test -f $PATCHBASE/README; then
   echo "1. Display Patch information"
   echo "2. Apply Patch"
   echo "3. Remove patch temporary files"
   echo "0. Exit"
   echo
   read -p "Select: " option
   case $option in
    1) disp_patch_readme; sub_menu patch; ;;
    2) apply_patch; sub_menu patch; ;;
    3) remove_patch; sub_menu patch; ;;
    *) exit; ;;
   esac
  fi

  if `ls $UPLOAD/[pP]atch.* > /dev/null 2>&1`; then
   patch_count > /dev/null 2>&1
   if test $patch_qty != 1; then
    echo "Only one patch file may be uploaded at a time"
    echo; read -p "Press Return to Continue" yn
    exit; fi
   patch_name=`ls $UPLOAD/[pP]atch.*`
   patch_name=`basename $patch_name`
   echo "1. Import patch file: $patch_name"
   echo "2. Delete patch file: $patch_name"
   echo "0. Exit"
   echo
   read -p "Select: " option
   case $option in
    1) import_patch; menu; ;;
    2) del_file $patch_name ; menu; ;;
    *) menu; ;;
   esac
  fi
 fi
 menu
}

del_file() {
 heading "Delete file"
 echo "Delete $1"
 yorn menu
 rm -f $UPLOAD/$1
 echo; echo "$1 Deleted"
 sleep 2
}

patch_count() {
 patch_qty=`ls $UPLOAD/[pP]atch.*|wc -l`
}

chk_tomcat() {
 if test -f $LOCK/tomcat5; then
  echo "Tomcat is running and must be stopped before continuing"
  echo; echo -n "Press Return to Continue"
  read yn
  $1; fi
}

get_patch_ver() {
 PATCH_VER=`grep ^PATCH_VER $PATCHBASE/settings.sh|cut -d\" -f2`
}

apply_patch() {
 heading "Apply Patch/Update"
 sh $PATCHBASE/install.sh 
}

remove_patch() {
 heading "Patch/Update management"
 echo "Removing temporary patch files. Once the files have been removed the patch will"
 echo "require uploading again using SFTP."
 yorn menu
 rm -rf $PATCHBASE/*
}

disp_patch_readme() {
 s_heading
 get_patch_ver
 echo "Patch Version: $PATCH_VER"
 echo
 more $PATCHBASE/README
 echo; read -p "Press Return to Continue" yn
}

import_patch() {
 s_heading
 patch_count > /dev/null 2>&1
 if test $patch_qty = 1; then
  patch_name=`ls $UPLOAD/[pP]atch.*`
  patch_name=`basename $patch_name`
  echo "An update file has been placed in the upload area."
  echo
  echo "Please wait whilst the patch file is checked"
  sleep 2
  process_patch
 else
  echo -n "Only one patch file may be uploaded at a time"
  sleep 2
 fi
}

process_patch() {
 mv $UPLOAD/$patch_name $PATCHBASE
 cd $PATCHBASE
# gpg $patch_name >> /var/log/cmi/patch.log 2>&1
 count=`echo $patch_name | wc -c`; count=`expr $count - 1`
 patch_name=`echo $patch_name|cut -c -$count`
 echo

 if test -f $patch_name; then
  echo "Patch file valid and now available."
  tar xfz $PATCHBASE/$patch_name
 else
  echo "The patch file is invalid."
 # mv $patch_name.gpg $UPLOAD/failed_$patch_name.gpg
 fi

 read -p "Press Return to Continue"
}

yorn() {
 echo
 read -p "Are you sure? (Yes/No): " yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y" ; then
  $1; fi
}

initialise
menu
